@@ -5,14 +5,23 @@ import logging
 import os
 from typing import Union
 
+from .cdr import DetailedCDRApi
 from .devices import DevicesApi
+from .events import EventsApi
 from .groups import GroupsApi
 from .licenses import LicensesApi
 from .locations import LocationsApi
+from .memberships import MembershipApi
+from .messages import MessagesApi
 from .organizations import OrganizationApi
 from .people import PeopleApi
 from .person_settings import PersonSettingsApi
+from .reports import ReportsApi
 from .rest import RestSession
+from .room_tabs import RoomTabsApi
+from .rooms import RoomsApi
+from .team_memberships import TeamMembershipsApi
+from .teams import TeamsApi
 from .telephony import TelephonyApi
 from .tokens import Tokens
 from .webhook import WebhookApi
@@ -23,7 +32,7 @@ from dataclasses import dataclass
 
 __all__ = ['WebexSimpleApi']
 
-__version__ = '1.7.1'
+__version__ = '1.9.0'
 
 log = logging.getLogger(__name__)
 
@@ -34,20 +43,38 @@ class WebexSimpleApi:
     The main API object
     """
 
+    #: CDR API :class:`cdr.DetailedCDRApi`
+    cdr: DetailedCDRApi
     #: devices API :class:`devices.DevicesApi`
     devices: DevicesApi
+    #: events API; :class:`events.EventsApi`
+    events: EventsApi
     #: groups API :class:`groups.GroupsApi`
     groups: GroupsApi
     #: Licenses API :class:`licenses.LicensesApi`
     licenses: LicensesApi
     #: Location API :class:`locations.LocationsApi`
     locations: LocationsApi
+    #: membership API :class:`memberships.MembershipApi`
+    membership: MembershipApi
+    #: Messages API :class:`messages.MessagesApi`
+    messages: MessagesApi
     #: organization settings API
     organizations: OrganizationApi
     #: Person settings API :class:`person_settings.PersonSettingsApi`
     person_settings: PersonSettingsApi
     #: People API :class:`people.PeopleApi`
     people: PeopleApi
+    #: Reports API :class:`reports.ReportsApi`
+    reports: ReportsApi
+    #: Rooms API :class:`rooms.RoomsApi`
+    rooms: RoomsApi
+    #: Room tabs API :class:`room_tabs.RoomTabsApi`
+    room_tabs: RoomTabsApi
+    #: Teams API :class:`teams.TeamsApi`
+    teams: TeamsApi
+    #: Team memberships API :class:`TeamMembershipsApi`
+    team_memberships: TeamMembershipsApi
     #: Telephony (features) API :class:`telephony.TelephonyApi`
     telephony: TelephonyApi
     #: Webhooks API :class:`webhook.WebhookApi`
@@ -79,13 +106,22 @@ class WebexSimpleApi:
             tokens = Tokens(access_token=tokens)
 
         session = RestSession(tokens=tokens, concurrent_requests=concurrent_requests)
+        self.cdr = DetailedCDRApi(session=session)
         self.devices = DevicesApi(session=session)
+        self.events = EventsApi(session=session)
         self.groups = GroupsApi(session=session)
         self.licenses = LicensesApi(session=session)
         self.locations = LocationsApi(session=session)
+        self.membership = MembershipApi(session=session)
+        self.messages = MessagesApi(session=session)
         self.organizations = OrganizationApi(session=session)
         self.person_settings = PersonSettingsApi(session=session)
         self.people = PeopleApi(session=session)
+        self.reports = ReportsApi(session=session)
+        self.rooms = RoomsApi(session=session)
+        self.room_tabs = RoomTabsApi(session=session)
+        self.teams = TeamsApi(session=session)
+        self.team_memberships = TeamMembershipsApi(session=session)
         self.telephony = TelephonyApi(session=session)
         self.webhook = WebhookApi(session=session)
         self.workspaces = WorkspacesApi(session=session)
