@@ -21,7 +21,6 @@ class Privacy(ApiModel):
     aa_naming_dialing_enabled: Optional[bool]
     #: When true phone status directory privacy will be enabled.
     enable_phone_status_directory_privacy: Optional[bool]
-    # TODO: documentation defect: called "monitoredAgents" in docs
     #: List of people that are being monitored.
     #: for updates IDs can be used directly instead of :class:`wxc_sdk.common.PersonPlaceAgent` objects
     monitoring_agents: Optional[list[Union[str, PersonPlaceAgent]]]
@@ -34,7 +33,7 @@ class PrivacyApi(PersonSettingsApiChild):
 
     feature = 'privacy'
 
-    def read(self, *, person_id: str, org_id: str = None) -> Privacy:
+    def read(self, person_id: str, org_id: str = None) -> Privacy:
         """
         Get a person's Privacy Settings
 
@@ -58,7 +57,7 @@ class PrivacyApi(PersonSettingsApiChild):
         data = self.get(ep, params=params)
         return Privacy.parse_obj(data)
 
-    def configure(self, *, person_id: str, settings: Privacy, org_id: str = None):
+    def configure(self, person_id: str, settings: Privacy, org_id: str = None):
         """
         Configure Call Waiting Settings for a Person
 
